import fs from "fs/promises";
import path from "path";
import ReactDOM from "react-dom/server";
import rehypeParse from "rehype-parse";
import rehypeStringify from "rehype-stringify";
import { unified } from "unified";
import { fileURLToPath } from "url";
import { build } from "vite";
import pluginRouter, { routerResolvedVirtualModuleId } from "./pluginRouter.js";
const ssrEntryName = "entryClientSsr.js";
const isChunk = (output) => output.type === "chunk";
function chunkCss(chunk) {
    if (!chunk.hasOwnProperty("viteMetadata")) {
        return [];
    }
    const chunkMeta = chunk.viteMetadata;
    var css = [];
    for (const cssAsset of chunkMeta.importedCss) {
        css.push(`/${cssAsset}`);
    }
    return css;
}
function findChunk(id, chunks) {
    for (const chunk of chunks) {
        if (isChunk(chunk) &&
            chunk.hasOwnProperty("facadeModuleId") &&
            chunk.facadeModuleId === id) {
            return chunk;
        }
    }
}
function serverHtml(css, module) {
    const initialCss = css
        .map((asset) => `<link rel="stylesheet" href="${asset}">`)
        .join("\n");
    const initialHtml = ReactDOM.renderToString(module());
    return `
<!DOCTYPE html>
<html>
<head>
${initialCss}
</head>
<body>
  <div id="root">
    ${initialHtml}
  </div>
  <script src="/${ssrEntryName}" type="module"></script>
</body>
</html>`;
}
export default function pluginBuild(moduleMap, rehypePlugins) {
    const ssrEntryDir = path.dirname(fileURLToPath(import.meta.url));
    const ssrEntryPath = path.join(ssrEntryDir, ssrEntryName);
    const processor = unified()
        .use(rehypeParse)
        .use(rehypePlugins)
        .use(rehypeStringify);
    var resolvedConfig;
    return {
        name: "vite-plugin-blog:build",
        apply: "build",
        config() {
            return {
                build: {
                    rollupOptions: {
                        input: [ssrEntryPath],
                        output: { entryFileNames: "[name].js" },
                    },
                },
            };
        },
        configResolved(config) {
            resolvedConfig = config;
        },
        async writeBundle(_output, bundle) {
            // Vite/Rollup notifies all `writeBundle` listeners, even if they're
            // not declared as plugins in the build. This check prevents an
            // infinite build when we build SSR later.
            if (resolvedConfig.build.ssr) {
                return;
            }
            const clientOutDir = path.join(resolvedConfig.root, resolvedConfig.build.outDir);
            const ssrOutDir = `${clientOutDir}-ssr`;
            // Build the SSR router, which will allow us to import modules and render
            // the initial HTML.
            const ssrBuild = await build({
                plugins: [pluginRouter(moduleMap)],
                build: {
                    outDir: ssrOutDir,
                    ssr: true,
                },
            });
            const ssrOutput = ssrBuild.output;
            const routerChunk = findChunk(routerResolvedVirtualModuleId, ssrOutput);
            if (!routerChunk) {
                throw new Error("Unable to locate router for initial HTML rendering");
            }
            const routerPath = path.join(ssrOutDir, routerChunk.fileName);
            const router = (await import(routerPath)).pages;
            for (const url in moduleMap) {
                const expectedId = path.join(resolvedConfig.root, moduleMap[url]);
                const moduleChunk = findChunk(expectedId, Object.values(bundle));
                const moduleCss = moduleChunk ? chunkCss(moduleChunk) : [];
                const module = (await router[url]()).default;
                const initialHtml = serverHtml(moduleCss, module);
                const html = String(await processor.process(initialHtml));
                const indexDir = [clientOutDir, url].join("/");
                await fs.mkdir(indexDir, { recursive: true });
                await fs.writeFile([indexDir, "index.html"].join("/"), html);
            }
        },
    };
}
